﻿' <auto-generated />
' Licensed to the .NET Foundation under one or more agreements.
' The .NET Foundation licenses this file to you under the MIT license.
' See the LICENSE file in the project root for more information.

' This was copied from https://github.com/dotnet/coreclr/blob/60f1e6265bd1039f023a82e0643b524d6aaf7845/src/System.Private.CoreLib/shared/System/Diagnostics/CodeAnalysis/NullableAttributes.cs
' and updated to have the scope of the attributes be internal.

Imports System
Imports System.Diagnostics

Namespace Global.System.Diagnostics.CodeAnalysis

    ''' <summary>Specifies that null is allowed as an input even if the corresponding type disallows it.</summary>
    <AttributeUsage(AttributeTargets.Field Or AttributeTargets.Parameter Or AttributeTargets.Property, Inherited:=False)>
    Friend NotInheritable Class AllowNullAttribute
        Inherits Attribute
    End Class

    ''' <summary>Specifies that null is disallowed as an input even if the corresponding type allows it.</summary>
    <AttributeUsage(AttributeTargets.Field Or AttributeTargets.Parameter Or AttributeTargets.Property, Inherited:=False)>
    Friend NotInheritable Class DisallowNullAttribute
        Inherits Attribute
    End Class

    ''' <summary>Specifies that an output may be null even if the corresponding type disallows it.</summary>
    <AttributeUsage(AttributeTargets.Field Or AttributeTargets.Parameter Or AttributeTargets.Property Or AttributeTargets.ReturnValue, Inherited:=False)>
    Friend NotInheritable Class MaybeNullAttribute
        Inherits Attribute
    End Class

    ''' <summary>Specifies that an output will not be null even if the corresponding type allows it.</summary>
    <AttributeUsage(AttributeTargets.Field Or AttributeTargets.Parameter Or AttributeTargets.Property Or AttributeTargets.ReturnValue, Inherited:=False)>
    Friend NotInheritable Class NotNullAttribute
        Inherits Attribute
    End Class

    ''' <summary>Specifies that when a method returns <see cref="ReturnValue"/>, the parameter may be null even if the corresponding type disallows it.</summary>
    <AttributeUsage(AttributeTargets.Parameter, Inherited:=False)>
    Friend NotInheritable Class MaybeNullWhenAttribute
        Inherits Attribute

        ''' <summary>Initializes the attribute with the specified return value condition.</summary>
        ''' <param name="returnValue">
        ''' The return value condition. If the method returns this value, the associated parameter may be null.
        ''' </param>
        Public Sub New(returnValue As Boolean)
            Me.ReturnValue = returnValue
        End Sub

        ''' <summary>Gets the return value condition.</summary>
        Public ReadOnly Property ReturnValue As Boolean
    End Class

    ''' <summary>Specifies that when a method returns <see cref="ReturnValue"/>, the parameter will not be null even if the corresponding type allows it.</summary>
    <AttributeUsage(AttributeTargets.Parameter, Inherited:=False)>
    Friend NotInheritable Class NotNullWhenAttribute
        Inherits Attribute

        ''' <summary>Initializes the attribute with the specified return value condition.</summary>
        ''' <param name="returnValue">
        ''' The return value condition. If the method returns this value, the associated parameter will not be null.
        ''' </param>
        Public Sub New(returnValue As Boolean)
            Me.ReturnValue = returnValue
        End Sub

        ''' <summary>Gets the return value condition.</summary>
        Public ReadOnly Property ReturnValue As Boolean
    End Class

    ''' <summary>Specifies that the output will be non-null if the named parameter is non-null.</summary>
    <AttributeUsage(AttributeTargets.Parameter Or AttributeTargets.Property Or AttributeTargets.ReturnValue, AllowMultiple:=True, Inherited:=False)>
    Friend NotInheritable Class NotNullIfNotNullAttribute
        Inherits Attribute

        ''' <summary>Initializes the attribute with the associated parameter name.</summary>
        ''' <param name="parameterName">
        ''' The associated parameter name.  The output will be non-null if the argument to the parameter specified is non-null.
        ''' </param>
        Public Sub New(parameterName As String)
            Me.ParameterName = parameterName
        End Sub

        ''' <summary>Gets the associated parameter name.</summary>
        Public ReadOnly Property ParameterName As String
    End Class

    ''' <summary>Applied to a method that will never return under any circumstance.</summary>
    <AttributeUsage(AttributeTargets.Method, Inherited:=False)>
    Friend NotInheritable Class DoesNotReturnAttribute
        Inherits Attribute
    End Class

    ''' <summary>Specifies that the method will not return if the associated Boolean parameter is passed the specified value.</summary>
    <AttributeUsage(AttributeTargets.Parameter, Inherited:=False)>
    Friend NotInheritable Class DoesNotReturnAttribute
        Inherits Attribute

        ''' <summary>Initializes the attribute with the specified parameter value.</summary>
        ''' <param name="parameterValue">
        ''' The condition parameter value. Code after the method will be considered unreachable by diagnostics if the argument to
        ''' the associated parameter matches this value.
        ''' </param>
        Public Sub New(parameterValue As Boolean)
            Me.ParameterValue = parameterValue
        End Sub

        ''' <summary>Gets the condition parameter value.</summary>
        Public ReadOnly Property ParameterValue As Boolean
    End Class

End Namespace
